define(function(require){

	var Animation = require("./Animation");
	
	return function(model, fieldName, name, node, cfg){
		//cfg object: { name: "", ... }
		
		//Enable animation
		var animate = ( cfg !== undefined && cfg.animate === true ),
			//Delay of animation
			delay = (animate && cfg.delay !== undefined) ? cfg.delay : 500;
	
		/**
		 * Applies a callback to accessors
		 */
		var applyCallback = function(getter, setter){
		
			//observable property
			var value = model[fieldName];
						
			if (value === undefined || value.isObservable === undefined)
				//not observable
				return;
			
			/**
			 * Maps a value, if specified in binding
			 */
			var mapValue = function(value){
				if (typeof(cfg.values) == "object")
					for (var field in cfg.values)
						if (field == value)
							return cfg.values[field];
				return value;
			};
			
			value.addListener(function(newValue, oldValue){
			
				//Map old/new value
				newValue = mapValue(newValue);
				oldValue = mapValue(oldValue);

				//console.log(newValue, oldValue);//XXX
				
				if (typeof(newValue) == "undefined")
					//undefined values not allowed
					return;
			
				if (animate && oldValue !== undefined)
					Animation(
						setter,
						+oldValue,
						+newValue,
						delay
					);
				else
					setter(newValue);
			});
			
			if (value() === undefined)
				//No value is defined for property
				//Take over value from svg
				value(getter());
			else
				//Value in model is specified
				//Apply value to svg
				setter(mapValue(value()));
		};
		
		switch (name) {
			case "rotate":
				//TODO: preserve translation and scale
				
				//Use center as default x/y offset
				var x = cfg.xOffset ? cfg.xOffset + (+node.getAttribute("x")) : (+node.getAttribute("x")) + node.getAttribute("width")/2,
					y = cfg.yOffset ? cfg.yOffset + (+node.getAttribute("y")) : (+node.getAttribute("y")) + node.getAttribute("height")/2;
				applyCallback(
					function(){ return node.getAttribute("transform"); },
					function(v){ node.setAttribute("transform", "rotate("+v+","+x+","+y+")"); }
				);
			break;
			case "text":
				var tspanEl;
				//Search tspan element
				for (var i=0; i<node.children.length; i++)
					if (node.children[i].tagName === "tspan")
						tspanEl = node.children[i];
						
				applyCallback(
					function(){ return tspanEl.textContent; },
					function(v){ tspanEl.textContent = v; }
				);
			break;
			case "xoffset":
				var base = node.x.baseVal.value;
				applyCallback(
					function(){ return node.x.baseVal.value - base; },
					function(v){ node.x.baseVal.value = base + v; }
				);
			break;
			case "yoffset":
				var base = node.y.baseVal.value;
				applyCallback(
					function(){ return node.y.baseVal.value - base; },
					function(v){ node.y.baseVal.value = base + v; }
				);
			break;
			case "width":
				var x = +node.getAttribute("x"),
					startWidth = +node.getAttribute("width"),
					//The scale to use (relative(default)/absolute)
					scale = cfg.mode === "absolute" ? 1 : +startWidth;
					
				if (cfg.direction === "left"){
					//Fixed position is right, direction is to the left
					applyCallback(
						function(){
							return +node.getAttribute("width")/scale;
						},
						function(v){
							node.setAttribute("width", v * scale);
							node.setAttribute("x", x + startWidth - v*scale);
						}
					);
				} else
					//default
					applyCallback(
						function(){ return +node.getAttribute("width")/scale; },
						function(v){ node.setAttribute("width", v * scale); }
					);
			break;
			case "height":
				var y = +node.getAttribute("y"),
					startHeight = +node.getAttribute("height"),
					//The scale to use (relative(default)/absolute)
					scale = cfg.mode === "absolute" ? 1 : +startHeight;
					
				if (cfg.direction === "up"){
					//Fixed position is bottom, direction is upwards
					applyCallback(
						function(){
							return +node.getAttribute("height")/scale;
						},
						function(v){
							node.setAttribute("height", v * scale);
							node.setAttribute("y", y + startHeight - v*scale);
						}
					);
				} else
					//default
					applyCallback(
						function(){ return +node.getAttribute("height")/scale; },
						function(v){ node.setAttribute("height", v * scale); }
					);
			break;
			case "visibility":
				applyCallback(
					function(){ return (node.getAttribute(name) !== "hidden"); },
					function(v){ node.setAttribute(name, v ? "visible" : "hidden"); }
				);
			break;
			default:
				applyCallback(
					function(){ return node.style[name]; },
					function(v){ node.style[name] = v; }
				);
		};
			
	};
});